import pytest

from clinvar_api import client, exceptions, models

FAKE_ID = "SUBxxx"
FAKE_TOKEN = "1234567890abcdefghijklmnopqrstuvwxyz"
FAKE_HEADERS = {
    "SP-API-KEY": FAKE_TOKEN,
}


def test_config_long_token():
    config = client.Config(auth_token="1234567890", use_testing=False, use_dryrun=False)
    assert str(config) == (
        "Config(auth_token='12345*****', use_testing=False, use_dryrun=False, presubmission_validation=True, verify_ssl=True)"
    )


def test_config_short_token():
    config = client.Config(auth_token="123", use_testing=False, use_dryrun=False)
    assert str(config) == (
        "Config(auth_token='***', use_testing=False, use_dryrun=False, presubmission_validation=True, verify_ssl=True)"
    )


def test_submit_data_success(requests_mock):
    requests_mock.register_uri(
        "POST",
        "https://submit.ncbi.nlm.nih.gov/api/v1/submissions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json={"id": "SUB999999"},
    )
    result = client.submit_data(
        models.SubmissionContainer(),
        config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False),
    )
    assert str(result) == "Created(id='SUB999999')"


def test_submit_data_failed(requests_mock):
    requests_mock.register_uri(
        "POST",
        "https://submit.ncbi.nlm.nih.gov/api/v1/submissions/",
        request_headers=FAKE_HEADERS,
        status_code=400,
        reason="Bad request",
        json={"message": "Submission is incorrect"},
    )
    with pytest.raises(exceptions.SubmissionFailed):
        client.submit_data(
            models.SubmissionContainer(),
            config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False),
        )


def test_retrieve_status_result():
    status_result = client.RetrieveStatusResult(
        status=models.SubmissionStatus(actions=[]), summaries={}
    )
    assert (
        str(status_result)
        == "RetrieveStatusResult(status=SubmissionStatus(actions=[]), summaries={})"
    )


def test_retrieve_status_success_no_extra_file(requests_mock, data_submission_submitted):
    requests_mock.register_uri(
        "GET",
        f"https://submit.ncbi.nlm.nih.gov/api/v1/submissions/{FAKE_ID}/actions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json=data_submission_submitted,
    )
    result = client.retrieve_status(
        FAKE_ID, config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
    )
    assert str(result).replace("tzlocal", "tzutc") == (
        "RetrieveStatusResult(status=SubmissionStatus(actions=[SubmissionStatusActions("
        "id='SUB999999-1', responses=[], status='submitted', target_db='clinvar', "
        "updated=datetime.datetime(2021, 3, 19, 17, 24, 24, 384085, tzinfo=tzutc()))]), summaries={})"
    )


def test_retrieve_status_success_with_extra_files(
    requests_mock, data_submission_processed, data_summary_response_processed
):
    requests_mock.register_uri(
        "GET",
        f"https://submit.ncbi.nlm.nih.gov/api/v1/submissions/{FAKE_ID}/actions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json=data_submission_processed,
    )
    requests_mock.register_uri(
        "GET",
        (
            "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx"
            "/sub999999-summary-report.json/?format=attachment"
        ),
        status_code=200,
        reason="OK",
        json=data_summary_response_processed,
    )
    result = client.retrieve_status(
        FAKE_ID, config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
    )
    assert str(result).replace("tzlocal", "tzutc") == (
        "RetrieveStatusResult(status=SubmissionStatus(actions=[SubmissionStatusActions(id='SUB999999-1', "
        "responses=[SubmissionStatusResponse(status='processed', files=[SubmissionStatusFile(url='"
        "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx/sub999999-summary-report.json/"
        "?format=attachment')], message=SubmissionStatusResponseMessage(error_code=None, severity='info', "
        'text=\'Your ClinVar submission processing status is "Success". Please find the details in the file '
        "referenced by actions[0].responses[0].files[0].url.'), objects=[])], status='processed', "
        "target_db='clinvar', updated=datetime.datetime(2021, 3, 24, 4, 22, 4, 101297, tzinfo=tzutc()))]), "
        "summaries={'https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx/sub999999-summary-report.json/"
        "?format=attachment': SummaryResponse(batch_processing_status=<BatchProcessingStatus.SUCCESS: "
        "'Success'>, batch_release_status=<BatchReleaseStatus.NOT_RELEASED: 'Not released'>, "
        "submission_date='2021-03-25', submission_name='SUB673156', total_count=3, total_errors=0, "
        "total_public=0, total_success=3, deletions=None, submissions=[SummaryResponseSubmission("
        "identifiers=SummaryResponseSubmissionIdentifiers(clinvar_local_key='"
        "adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1', clinvar_accession='SCV000839746', "
        "local_id='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1', "
        "local_key='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success1'), processing_status='Success', "
        "clinvar_accession_version=None, errors=None, release_date=None, release_status=None), "
        "SummaryResponseSubmission(identifiers=SummaryResponseSubmissionIdentifiers("
        "clinvar_local_key='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2', "
        "clinvar_accession='SCV000839747', local_id='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2', "
        "local_key='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success2'), processing_status='Success', "
        "clinvar_accession_version=None, errors=None, release_date=None, release_status=None), "
        "SummaryResponseSubmission(identifiers=SummaryResponseSubmissionIdentifiers("
        "clinvar_local_key='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3', "
        "clinvar_accession='SCV000839748', local_id='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3', "
        "local_key='adefc5ed-7d59-4119-8b3d-07dcdc504c09_success3'), processing_status='Success', "
        "clinvar_accession_version=None, errors=None, release_date=None, release_status=None)], "
        "total_delete_count=None, total_deleted=None, total_delete_errors=None, total_delete_success=None)})"
    )


def test_retrieve_status_failed_initial_request(requests_mock):
    requests_mock.register_uri(
        "GET",
        f"https://submit.ncbi.nlm.nih.gov/api/v1/submissions/{FAKE_ID}/actions/",
        request_headers=FAKE_HEADERS,
        status_code=401,
        reason="Unauthorized",
        json={"message": "No valid API key provided"},
    )

    with pytest.raises(exceptions.QueryFailed):
        client.retrieve_status(
            FAKE_ID, config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
        )


def test_retrieve_status_failed_extra_request(requests_mock, data_submission_processed):
    requests_mock.register_uri(
        "GET",
        f"https://submit.ncbi.nlm.nih.gov/api/v1/submissions/{FAKE_ID}/actions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json=data_submission_processed,
    )
    requests_mock.register_uri(
        "GET",
        (
            "https://dsubmit.ncbi.nlm.nih.gov/api/2.0/files/xxxxxxxx"
            "/sub999999-summary-report.json/?format=attachment"
        ),
        status_code=404,
        reason="Not Found",
    )

    with pytest.raises(exceptions.QueryFailed):
        client.retrieve_status(
            FAKE_ID, config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
        )


def test_client_submit_success(requests_mock):
    requests_mock.register_uri(
        "POST",
        "https://submit.ncbi.nlm.nih.gov/api/v1/submissions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json={"id": "SUB999999"},
    )
    client_obj = client.Client(
        config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
    )
    result = client_obj.submit_data(models.SubmissionContainer())
    assert str(result) == "Created(id='SUB999999')"


def test_client_retrieve_status_success_no_extra_file(requests_mock, data_submission_submitted):
    requests_mock.register_uri(
        "GET",
        f"https://submit.ncbi.nlm.nih.gov/api/v1/submissions/{FAKE_ID}/actions/",
        request_headers=FAKE_HEADERS,
        status_code=200,
        reason="OK",
        json=data_submission_submitted,
    )
    client_obj = client.Client(
        config=client.Config(auth_token=FAKE_TOKEN, presubmission_validation=False)
    )
    result = client_obj.retrieve_status(FAKE_ID)
    assert str(result).replace("tzlocal", "tzutc") == (
        "RetrieveStatusResult(status=SubmissionStatus(actions=[SubmissionStatusActions("
        "id='SUB999999-1', responses=[], status='submitted', target_db='clinvar', "
        "updated=datetime.datetime(2021, 3, 19, 17, 24, 24, 384085, tzinfo=tzutc()))]), summaries={})"
    )



"""Test ``clinvar_api.msg`` with unit tests."""

import datetime

from clinvar_api import msg


def test_created_construction():
    msg.Created(id="test-id")


def test_error_construction():
    msg.Error(message="fake-message")


def test_submission_status_file_construction():
    msg.SubmissionStatusFile(url="http://example.com")


def test_submission_status_object_content_construction():
    msg.SubmissionStatusObjectContent(
        clinvarProcessingStatus="In processing",
        clinvarReleaseStatus="Not released",
    )


def test_submission_status_object_construction():
    msg.SubmissionStatusObject(
        accession="accession",
        content=msg.SubmissionStatusObjectContent(
            clinvarProcessingStatus="In processing",
            clinvarReleaseStatus="Not released",
        ),
        targetDb="clinvar",
    )
    msg.SubmissionStatusObject(
        accession=None,
        content=msg.SubmissionStatusObjectContent(
            clinvarProcessingStatus="In processing",
            clinvarReleaseStatus="Not released",
        ),
        targetDb="clinvar",
    )


def test_submission_status_response_message_construction():
    msg.SubmissionStatusResponseMessage(
        errorCode="error-code", severity="fake-severity", text="fake text"
    )
    msg.SubmissionStatusResponseMessage(errorCode=None, severity="fake-severity", text="fake text")


def test_submission_status_response_construction():
    msg.SubmissionStatusResponse(
        status=None,
        files=[msg.SubmissionStatusFile(url="http://example.com")],
        message=msg.SubmissionStatusResponseMessage(
            errorCode="error-code", severity="fake-severity", text="fake text"
        ),
        objects=[
            msg.SubmissionStatusObject(
                accession=None,
                content=msg.SubmissionStatusObjectContent(
                    clinvarProcessingStatus="In processing",
                    clinvarReleaseStatus="Not released",
                ),
                targetDb="clinvar",
            )
        ],
    )
    msg.SubmissionStatusResponse(
        status="fake-status",
        files=[msg.SubmissionStatusFile(url="http://example.com")],
        message=msg.SubmissionStatusResponseMessage(
            errorCode="error-code", severity="fake-severity", text="fake text"
        ),
        objects=[
            msg.SubmissionStatusObject(
                accession=None,
                content=msg.SubmissionStatusObjectContent(
                    clinvarProcessingStatus="In processing",
                    clinvarReleaseStatus="Not released",
                ),
                targetDb="clinvar",
            )
        ],
    )


def test_submission_status_actions_construction():
    msg.SubmissionStatusActions(
        id="fake-id",
        responses=[
            msg.SubmissionStatusResponse(
                status=None,
                files=[msg.SubmissionStatusFile(url="http://example.com")],
                message=msg.SubmissionStatusResponseMessage(
                    errorCode="error-code", severity="fake-severity", text="fake text"
                ),
                objects=[
                    msg.SubmissionStatusObject(
                        accession=None,
                        content=msg.SubmissionStatusObjectContent(
                            clinvarProcessingStatus="In processing",
                            clinvarReleaseStatus="Not released",
                        ),
                        targetDb="clinvar",
                    )
                ],
            )
        ],
        status="the-status",
        targetDb="clinvar",
        updated=datetime.datetime.now(),
    )
    msg.SubmissionStatusActions(
        id="fake-id",
        responses=[],
        status="the-status",
        targetDb="clinvar",
        updated=datetime.datetime.now(),
    )


def test_submission_status_construction():
    msg.SubmissionStatus(actions=[])
    msg.SubmissionStatus(
        actions=[
            msg.SubmissionStatusActions(
                id="fake-id",
                responses=[],
                status="the-status",
                targetDb="clinvar",
                updated=datetime.datetime.now(),
            )
        ]
    )


def test_summary_response_error_input_construction():
    msg.SummaryResponseErrorInput(value="value", field="field")
    msg.SummaryResponseErrorInput(value="value", field=None)


def test_summary_response_error_output_error_construction():
    msg.SummaryResponseErrorOutputError(userMessage="the message")


def test_summary_response_error_output_construction():
    msg.SummaryResponseErrorOutput(
        errors=[msg.SummaryResponseErrorOutputError(userMessage="the message")]
    )
    msg.SummaryResponseErrorOutput(errors=[])


def test_summary_response_error_construction():
    msg.SummaryResponseError(
        input=[msg.SummaryResponseErrorInput(value="value", field="field")],
        output=msg.SummaryResponseErrorOutput(errors=[]),
    )
    msg.SummaryResponseError(input=[], output=msg.SummaryResponseErrorOutput(errors=[]))


def test_summary_response_deletion_identifier_construction():
    msg.SummaryResponseDeletionIdentifier(
        clinvarAccession="the-accession",
        clinvarLocalKey="the-local-key",
    )
    msg.SummaryResponseDeletionIdentifier(
        clinvarAccession="the-accession",
        clinvarLocalKey=None,
    )


def test_summary_response_deletion_construction():
    msg.SummaryResponseDeletion(
        identifiers=msg.SummaryResponseDeletionIdentifier(
            clinvarAccession="the-accession",
        ),
        processingStatus="processing-status",
        deleteDate="2022-01-01",
        deleteStatus="delete-status",
        errors=[
            msg.SummaryResponseError(input=[], output=msg.SummaryResponseErrorOutput(errors=[]))
        ],
    )
    msg.SummaryResponseDeletion(
        identifiers=msg.SummaryResponseDeletionIdentifier(
            clinvarAccession="the-accession",
        ),
        processingStatus="processing-status",
        deleteDate=None,
        deleteStatus=None,
        errors=None,
    )


def test_summary_response_submission_identifier_construction():
    msg.SummaryResponseSubmissionIdentifiers(
        clinvarLocalKey="local-key",
        clinvarAccession="the-accession",
        localID="local-id",
        localKey="local-key",
    )
    msg.SummaryResponseSubmissionIdentifiers(
        clinvarLocalKey="local-key",
        clinvarAccession=None,
        localID=None,
        localKey=None,
    )


def test_summary_response_submission_construction():
    msg.SummaryResponseSubmission(
        identifiers=msg.SummaryResponseSubmissionIdentifiers(clinvarLocalKey="local-key"),
        processingStatus="processing-status",
        clinvarAccessionVersion="accession-version",
        errors=[
            msg.SummaryResponseError(input=[], output=msg.SummaryResponseErrorOutput(errors=[]))
        ],
        releaseDate="2022-01-01",
    )
    msg.SummaryResponseSubmission(
        identifiers=msg.SummaryResponseSubmissionIdentifiers(clinvarLocalKey="local-key"),
        processingStatus="processing-status",
        clinvarAccessionVersion=None,
        errors=None,
        releaseDate=None,
    )


def test_summary_response_construction():
    msg.SummaryResponse(
        batchProcessingStatus="status",
        batchReleaseStatus="released",
        submissionDate="2022-01-01",
        submissionName="submission-name",
        totalCount=10,
        totalErrors=10,
        totalPublic=10,
        totalSuccess=10,
        deletions=[
            msg.SummaryResponseDeletion(
                identifiers=msg.SummaryResponseDeletionIdentifier(
                    clinvarAccession="the-accession",
                ),
                processingStatus="processing-status",
            )
        ],
        submissions=[
            msg.SummaryResponseSubmission(
                identifiers=msg.SummaryResponseSubmissionIdentifiers(clinvarLocalKey="local-key"),
                processingStatus="processing-status",
            )
        ],
        totalDeleteCount=10,
        totalDeleted=10,
        totalDeleteErrors=10,
        totalDeleteSuccess=10,
    )
    msg.SummaryResponse(
        batchProcessingStatus="status",
        batchReleaseStatus="released",
        submissionDate="2022-01-01",
        submissionName="submission-name",
        totalCount=10,
        totalErrors=10,
        totalPublic=10,
        totalSuccess=10,
        deletions=None,
        submissions=None,
        totalDeleteCount=None,
        totalDeleted=None,
        totalDeleteErrors=None,
        totalDeleteSuccess=None,
    )


def test_sumbmission_clinvar_deletetion_accession_set_construction():
    msg.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason="whatever",
    )
    msg.SubmissionClinvarDeletionAccessionSet(
        accession="accession",
        reason=None,
    )


def test_submission_clinvar_deletion_construction():
    msg.SubmissionClinvarDeletion(
        accessionSet=[
            msg.SubmissionClinvarDeletionAccessionSet(
                accession="accession",
                reason="whatever",
            )
        ]
    )
    msg.SubmissionClinvarDeletion(accessionSet=[])


def test_submission_chromosome_coordinates_construction():
    msg.SubmissionChromosomeCoordinates(
        accession="accession",
        alternateAllele="A",
        assembly=msg.Assembly.GRCH37,
        chromosome=msg.Chromosome.CHR12,
        innerStart=100,
        innerStop=200,
        outerStart=90,
        outerStop=210,
        referenceAllele="C",
        start=100,
        stop=200,
        variantLength=10,
    )
    msg.SubmissionChromosomeCoordinates(
        assembly=msg.Assembly.GRCH37,
        chromosome=msg.Chromosome.CHR12,
        start=100,
        stop=100,
        referenceAllele="C",
        alternateAllele="A",
        variantLength=1,
    )


def test_submission_variant_gene_construction():
    msg.SubmissionVariantGene(
        id="some-id",
        symbol="some-symbol",
    )
    msg.SubmissionVariantGene()


def test_submission_variant_construction():
    msg.SubmissionVariant(
        chromosomeCoordinates=msg.SubmissionChromosomeCoordinates(
            assembly=msg.Assembly.GRCH37,
            chromosome=msg.Chromosome.CHR12,
            start=100,
            stop=100,
            referenceAllele="C",
            alternateAllele="A",
            variantLength=1,
        ),
        copyNumber="2",
        gene=[msg.SubmissionVariantGene()],
        hgvs="hgvs-string",
        referenceCopyNumber="2",
        variantType=msg.VariantType.DELETION,
    )
    msg.SubmissionVariant()


def test_sumbmission_variant_set_construction():
    msg.SubmissionVariantSet(variant=[msg.SubmissionVariant()])
    msg.SubmissionVariantSet(variant=[])


def test_submission_phase_unknown_set_construction():
    msg.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[msg.SubmissionVariant()],
    )
    msg.SubmissionPhaseUnknownSet(
        hgvs="hgvs-string",
        variants=[],
    )


def test_submission_clinical_feature_construction():
    msg.SubmissionClinicalFeature(
        clinicalFeaturesAffectedStatus="affected-status",
        db="HP",
        id="the-id",
        name="same-name",
    )


def test_submission_observed_in_construction():
    msg.SubmissionObservedIn(
        affectedStatus="affected",
        alleleOrigin="germline",
        collectionMethod="clinical testing",
        clinicalFeatures=[
            msg.SubmissionClinicalFeature(
                clinicalFeaturesAffectedStatus="affected-status",
                db="HP",
                id="the-id",
                name="same-name",
            )
        ],
        clinicalFeaturesComment="some comment",
        numberOfIndividuals=1,
        structVarMethodType="some-type",
    )
    msg.SubmissionObservedIn(
        affectedStatus="affected",
        alleleOrigin="germline",
        collectionMethod="clinical testing",
    )


def test_submission_haplotype_set_construction():
    msg.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[],
        starAlleleName="the-name",
    )
    msg.SubmissionHaplotypeSet(
        hgvs="the-hgvs",
        variants=[msg.SubmissionVariant()],
        starAlleleName="the-name",
    )


def test_submission_distinct_chromosome_set_construction():
    msg.SubmissionDistinctChromosomesSet(
        hgvs="some-hgvs",
        variants=[
            msg.SubmissionVariant(),
            msg.SubmissionVariant(),
        ],
    )


def test_submission_haplotype_sets_construction():
    msg.SubmissionHaplotypeSets(
        haplotypeSet=msg.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[msg.SubmissionVariant()],
            starAlleleName="the-name",
        ),
        haplotypeSingleVariantSet=msg.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[msg.SubmissionVariant()],
            starAlleleName="the-name",
        ),
    )
    msg.SubmissionHaplotypeSets()


def test_submission_diplotype_set_construction():
    msg.SubmissionDiplotypeSet(
        haplotypeSets=[msg.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        starAlleleName="star-allele-name",
    )
    msg.SubmissionDiplotypeSet(
        haplotypeSets=[msg.SubmissionHaplotypeSets()],
        hgvs="the-hgvs",
        starAlleleName=None,
    )


def test_submission_citation_construction():
    msg.SubmissionCitation(
        db=msg.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    )
    msg.SubmissionCitation(
        db=None,
        id=None,
        url=None,
    )


def test_submission_assertion_criteria_construction():
    msg.SubmissionAssertionCriteria(
        db=msg.CitationDb.PMC,
        id="PMC123",
        url="https://example.com",
    )


def test_submission_condition_construction():
    msg.SubmissionCondition(
        db=msg.ConditionDb.HP,
        id="some-id",
        name="some-name",
    )
    msg.SubmissionCondition(
        db=None,
        id=None,
        name=None,
    )


def test_submission_drug_response_construction():
    msg.SubmissionDrugResponse(
        db=msg.ConditionDb.HP,
        drugName="some drug",
        id="some-id",
        condition=[msg.SubmissionCondition()],
    )
    msg.SubmissionDrugResponse(
        db=None,
        drugName=None,
        id=None,
        condition=None,
    )


def test_submission_condition_set_construction():
    msg.SubmissionConditionSet(
        condition=[msg.SubmissionCondition()],
        drugResponse=[msg.SubmissionDrugResponse()],
    )
    msg.SubmissionConditionSet(
        condition=None,
        drugResponse=None,
    )


def test_submission_compound_heterozygote_set_variant_set_construction():
    msg.SubmissionCompoundHeterozygoteSetVariantSet(
        variantSet=msg.SubmissionVariantSet(variant=[]),
    )
    msg.SubmissionCompoundHeterozygoteSetVariantSet(
        variantSet=None,
    )


def test_submission_compound_heterozygote_set_construction():
    msg.SubmissionCompoundHeterozygoteSet(
        hgvs="hgvs",
        variantSets=[
            msg.SubmissionCompoundHeterozygoteSetVariantSet(),
            msg.SubmissionCompoundHeterozygoteSetVariantSet(),
        ],
    )


def test_submission_clinical_significance_construction():
    msg.SubmissionClinicalSignificance(
        clinicalSignificanceDescription=msg.ClinicalSignificanceDescription.PATHOGENIC,
        citation=[
            msg.SubmissionCitation(
                db=msg.CitationDb.PMC,
                id="PMC123",
                url="https://example.com",
            )
        ],
        comment="some comment",
        customAssertionScore=42.0,
        dateLastEvaluated="2022-01-01",
        explanationOfDrugResponse="some explanation",
        explanationOfOtherClinicalSignificance="2022-01-01",
        modeOfInheritance=msg.ModeOfInheritance.AUTOSOMAL_DOMINANT_INHERITANCE,
    )
    msg.SubmissionClinicalSignificance(
        clinicalSignificanceDescription=msg.ClinicalSignificanceDescription.PATHOGENIC,
    )


def test_submission_clinvar_submission_construction():
    msg.SubmissionClinvarSubmission(
        clinicalSignificance=msg.SubmissionClinicalSignificance(
            clinicalSignificanceDescription=msg.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        conditionSet=msg.SubmissionConditionSet(),
        observedIn=[
            msg.SubmissionObservedIn(
                affectedStatus="affected",
                alleleOrigin="germline",
                collectionMethod="clinical testing",
            )
        ],
        recordStatus="record-status",
        clinvarAccession="some-accession",
        compoundHeterozygoteSet=msg.SubmissionCompoundHeterozygoteSet(
            hgvs="hgvs",
            variantSets=[
                msg.SubmissionCompoundHeterozygoteSetVariantSet(),
                msg.SubmissionCompoundHeterozygoteSetVariantSet(),
            ],
        ),
        diplotypeSet=msg.SubmissionDiplotypeSet(
            haplotypeSets=[msg.SubmissionHaplotypeSets()],
            hgvs="the-hgvs",
        ),
        distinctChromosomesSet=msg.SubmissionDistinctChromosomesSet(
            hgvs="some-hgvs",
            variants=[
                msg.SubmissionVariant(),
                msg.SubmissionVariant(),
            ],
        ),
        haplotypeSet=msg.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[],
            starAlleleName="the-name",
        ),
        haplotypeSingleVariantSet=msg.SubmissionHaplotypeSet(
            hgvs="the-hgvs",
            variants=[msg.SubmissionVariant()],
            starAlleleName="the-name",
        ),
        localID="local-id",
        localKey="local-key",
        phaseUnknownSet=msg.SubmissionPhaseUnknownSet(
            hgvs="hgvs-string",
            variants=[],
        ),
        variantSet=msg.SubmissionVariantSet(variant=[]),
    )
    msg.SubmissionClinvarSubmission(
        clinicalSignificance=msg.SubmissionClinicalSignificance(
            clinicalSignificanceDescription=msg.ClinicalSignificanceDescription.PATHOGENIC,
        ),
        conditionSet=msg.SubmissionConditionSet(),
        observedIn=[
            msg.SubmissionObservedIn(
                affectedStatus="affected",
                alleleOrigin="germline",
                collectionMethod="clinical testing",
            )
        ],
        recordStatus="record-status",
    )


def test_submission_container_construction():
    msg.SubmissionContainer(
        behalfOrgID=123,
        clinvarDeletion=msg.SubmissionClinvarDeletion(
            accessionSet=[
                msg.SubmissionClinvarDeletionAccessionSet(
                    accession="accession",
                    reason="whatever",
                )
            ]
        ),
        clinvarSubmissionReleaseStatus=None,
        submissionName="some-name",
    )
    msg.SubmissionContainer(
        assertionCriteria=msg.SubmissionAssertionCriteria(
            db=msg.CitationDb.PMC,
            id="PMC123",
            url="https://example.com",
        ),
        behalfOrgID=123,
        clinvarDeletion=None,
        clinvarSubmissionReleaseStatus="release-status",
        clinvarSubmission=[
            msg.SubmissionClinvarSubmission(
                clinicalSignificance=msg.SubmissionClinicalSignificance(
                    clinicalSignificanceDescription=msg.ClinicalSignificanceDescription.PATHOGENIC,
                ),
                conditionSet=msg.SubmissionConditionSet(),
                observedIn=[
                    msg.SubmissionObservedIn(
                        affectedStatus="affected",
                        alleleOrigin="germline",
                        collectionMethod="clinical testing",
                    )
                ],
                recordStatus="record-status",
            )
        ],
        submissionName="some-name",
    )



from unittest.mock import MagicMock, patch

from click.testing import CliRunner
import pytest

from clinvar_this import batches, cli, config, exceptions


def test_call_main_help():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["--help"])
    assert result.exit_code == 0


def test_call_main_verbose():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["--verbose"])
    assert result.exit_code != 0


def test_call_config():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["config"])
    assert result.exit_code == 0


def test_call_config_get():
    with patch(
        "clinvar_this.cli.load_config",
        MagicMock(return_value=config.Config(profile="default", auth_token="fake")),
    ):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "get", "auth_token"])
    assert b"fake" in result.stdout_bytes
    assert result.exit_code == 0


def test_call_config_dump(fs_config):
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["config", "dump"])
    assert b"fake" in result.stdout_bytes
    assert result.exit_code == 0


def test_call_config_set_success():
    mock_save_config = MagicMock()
    with patch(
        "clinvar_this.cli.load_config",
        MagicMock(return_value=config.Config(profile="default", auth_token="fake")),
    ), patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "auth_token", "xxx"])
    mock_save_config.assert_called_once_with(
        config.Config(profile="default", auth_token="xxx"), "default"
    )
    assert result.exit_code == 0


def test_call_config_set_success_though_missing_config():
    mock_save_config = MagicMock()
    with patch(
        "clinvar_this.cli.load_config", MagicMock(side_effect=exceptions.ConfigFileMissingException)
    ), patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "auth_token", "xxx"])
    mock_save_config.assert_called_once_with(
        config.Config(profile="default", auth_token="xxx"), "default"
    )
    assert result.exit_code == 0


def test_call_config_set_fail_invalid_name():
    mock_save_config = MagicMock()
    with patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "xxx", "xxx"])
    # assert b"Invalid value" in result.stderr_bytes
    assert result.exit_code != 0


@pytest.mark.parametrize(
    "force,struc_var",
    [
        (True, False),
        (False, False),
        (True, True),
        (False, True),
    ],
)
def test_call_batch_export(fs_config, monkeypatch, force, struc_var):
    """Unit test of ``batch export`` with both sequence and structural variant.

    The test stubs out call of ``batches.list`` and checks results.
    """
    mock_export = MagicMock()
    monkeypatch.setattr(batches, "export", mock_export)

    args = ["batch", "export", "batch-name", "out-tsv"]
    if force:
        args.append("--force")
    if struc_var:
        args.append("--struc-var")

    runner = CliRunner()
    result = runner.invoke(cli.cli, args)

    mock_export.assert_called_once()
    assert len(mock_export.call_args.args) == 5
    assert len(mock_export.call_args.kwargs) == 0
    assert (
        str(mock_export.call_args.args[0])
        == "Config(profile='default', auth_token='****', verify_ssl=True)"
    )
    assert mock_export.call_args.args[1] == "batch-name"
    assert mock_export.call_args.args[2] == "out-tsv"
    assert mock_export.call_args.args[3] == force
    assert mock_export.call_args.args[4] == struc_var
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "name,metadata",
    [
        (None, None),
        ("foo", None),
        (None, ("A=a", "B=b")),
    ],
)
def test_call_batch_import(fs_config, monkeypatch, name, metadata):
    """Unit test of ``batch import``.

    The test stubs out call of ``batches.import_`` and checks results.
    """
    mock_import = MagicMock()
    monkeypatch.setattr(batches, "import_", mock_import)

    mock_gen_name = MagicMock()
    mock_gen_name.return_value = "generated"
    monkeypatch.setattr(batches, "gen_name", mock_gen_name)

    args = ["batch", "import", "input-tsv"]
    if name:
        args += ["--name", name]
    for m in metadata or ():
        args += ["--metadata", m]

    runner = CliRunner()
    result = runner.invoke(cli.cli, args)

    mock_import.assert_called_once()
    assert len(mock_import.call_args.args) == 4
    assert len(mock_import.call_args.kwargs) == 0
    assert (
        str(mock_import.call_args.args[0])
        == "Config(profile='default', auth_token='****', verify_ssl=True)"
    )
    assert mock_import.call_args.args[1] == (name if name else "generated")
    assert mock_import.call_args.args[2] == "input-tsv"
    assert mock_import.call_args.args[3] == (metadata if metadata else ())
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "name,metadata",
    [
        ("other", None),
        ("foo", ("A=a", "B=b")),
    ],
)
def test_call_batch_update_metadata(fs_config, monkeypatch, name, metadata):
    """Unit test of ``batch update-metadata``, stubs out call of ``batches.update_metadata`` and checks results."""
    mock_update_metadata = MagicMock()
    monkeypatch.setattr(batches, "update_metadata", mock_update_metadata)

    args = ["batch", "update-metadata", name]
    if metadata:
        args += metadata

    runner = CliRunner()
    result = runner.invoke(cli.cli, args)

    mock_update_metadata.assert_called_once()
    assert len(mock_update_metadata.call_args.args) == 3
    assert len(mock_update_metadata.call_args.kwargs) == 0
    assert (
        str(mock_update_metadata.call_args.args[0])
        == "Config(profile='default', auth_token='****', verify_ssl=True)"
    )
    assert mock_update_metadata.call_args.args[1] == name
    assert mock_update_metadata.call_args.args[2] == (metadata if metadata else ())
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "dry_run,use_testing",
    [
        (False, False),
        (False, True),
        (True, False),
    ],
)
def test_call_batch_submit(fs_config, monkeypatch, dry_run, use_testing):
    """Unit test of ``batch submit``, stubs out call of ``batches.submit`` and checks results."""
    mock_submit = MagicMock()
    monkeypatch.setattr(batches, "submit", mock_submit)

    args = ["batch", "submit", "name"]
    if dry_run:
        args.append("--dry-run")
    if use_testing:
        args.append("--use-testing")

    runner = CliRunner()
    result = runner.invoke(cli.cli, args)

    mock_submit.assert_called_once()
    assert len(mock_submit.call_args.args) == 2
    assert len(mock_submit.call_args.kwargs) == 2
    assert (
        str(mock_submit.call_args.args[0])
        == "Config(profile='default', auth_token='****', verify_ssl=True)"
    )
    assert mock_submit.call_args.args[1] == "name"
    assert mock_submit.call_args.kwargs == {"dry_run": dry_run, "use_testing": use_testing}
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "use_testing",
    [True, False],
)
def test_call_batch_retrieve(fs_config, monkeypatch, use_testing):
    """Unit test of ``batch retrieve``, stubs out call of ``batches.retrieve`` and checks results."""
    mock_retrieve = MagicMock()
    monkeypatch.setattr(batches, "retrieve", mock_retrieve)

    args = ["batch", "retrieve", "name"]
    if use_testing:
        args.append("--use-testing")

    runner = CliRunner()
    result = runner.invoke(cli.cli, args)

    mock_retrieve.assert_called_once()
    assert len(mock_retrieve.call_args.args) == 2
    assert len(mock_retrieve.call_args.kwargs) == 1
    assert (
        str(mock_retrieve.call_args.args[0])
        == "Config(profile='default', auth_token='****', verify_ssl=True)"
    )
    assert mock_retrieve.call_args.args[1] == "name"
    assert mock_retrieve.call_args.kwargs == {"use_testing": use_testing}
    assert result.exit_code == 0



from clinvar_api import common


def test_clean_for_json_bool():
    assert common.clean_for_json(True) is True


def test_clean_for_json_int():
    assert common.clean_for_json(1) == 1


def test_clean_for_json_float():
    assert common.clean_for_json(1.0) == 1.0


def test_clean_for_json_str():
    assert common.clean_for_json("x") == "x"


def test_clean_for_json_list():
    assert common.clean_for_json([1, 2, 3, 4]) == [1, 2, 3, 4]
    assert common.clean_for_json([[1], [2]]) == [[1], [2]]


def test_clean_for_json_dict():
    assert common.clean_for_json({"key": "value"}) == {"key": "value"}
    assert common.clean_for_json({"key": "value", "none": None}) == {"key": "value"}
    assert common.clean_for_json([{"key": "value", "none": None}]) == [{"key": "value"}]
    assert common.clean_for_json({"d": {"key": "value", "none": None}}) == {"d": {"key": "value"}}



"""Smoke test for the CLI to ClinVar XML parsing."""

import gzip
import os.path

from click.testing import CliRunner
import pytest

from clinvar_this import cli


@pytest.mark.parametrize(
    "fname_in,fname_out",
    [("one_record.xml", "one_record.jsonl"), ("one_record.xml.gz", "one_record.jsonl.gz")],
)
def test_cli_xml_to_json(fname_in, fname_out, tmp_path, snapshot):
    runner = CliRunner()
    in_path = os.path.dirname(__file__) + f"/data/{fname_in}"
    out_path = f"{tmp_path}/{fname_out}"
    result = runner.invoke(cli.cli, ["data", "xml-to-jsonl", in_path, out_path])
    assert result.exit_code == 0

    if fname_out.endswith(".gz"):
        output_f = gzip.open(out_path, "rt")
    else:
        output_f = open(out_path, "rt")
    with output_f:
        snapshot.assert_match(output_f.read(), "one_record.jsonl")


def test_cli_xml_to_json_stdin_stdout(snapshot):
    in_path = os.path.dirname(__file__) + "/data/one_record.xml"
    with open(in_path, "rt") as inputf:
        stdin_txt = inputf.read()

    runner = CliRunner(mix_stderr=False)
    result = runner.invoke(cli.cli, ["data", "xml-to-jsonl", "-", "-"], input=stdin_txt)
    assert result.exit_code == 0

    snapshot.assert_match(result.output, "one_record.jsonl")


@pytest.mark.parametrize(
    "fname_in",
    [
        "ex_additional_submitters.xml",
        "ex_attribute_set.xml",
        "ex_custom_score.xml",
        "ex_indication.xml",
        "ex_missense.xml",
        "ex_replaces.xml",
        "ex_review_status_ns.xml",
        "ex_study_description.xml",
        "ex_with_ethnicity.xml",
        "ex_with_hpo.xml",
    ],
)
def test_convert_snapshot_to_jsonl(fname_in, tmp_path, snapshot):
    runner = CliRunner()
    in_path = os.path.dirname(__file__) + f"/data/{fname_in}"
    out_path = f"{tmp_path}/out.jsonl"
    result = runner.invoke(cli.cli, ["data", "xml-to-jsonl", in_path, out_path])
    assert result.exit_code == 0

    with open(out_path, "rt") as output_f:
        snapshot.assert_match(output_f.read(), "out.jsonl")



import os

import pytest

from clinvar_this import config


@pytest.fixture
def fs_config(fs):
    """Fake file system with fake config"""
    fs.create_file(
        os.path.expanduser("~/.config/clinvar-this/config.toml"),
        contents='[default]\nauth_token = "fake"',
    )
    yield fs


@pytest.fixture
def app_config():
    yield config.Config(
        profile="default",
        auth_token="fake_auth_token",
        verify_ssl=False,
    )



from clinvar_data import class_by_freq


def test_smoke_test_run(tmpdir, snapshot):
    path_input = "tests/clinvar_data/data/records_with_gmaf.jsonl"
    path_output = f"{tmpdir}/output.jsonl"
    class_by_freq.run_report(path_input, path_output, thresholds=class_by_freq.DEFAULT_THRESHOLDS)

    with open(path_output, "rt") as outputf:
        snapshot.assert_match(outputf.read(), "output.jsonl")
